@@ -11,6 +11,46 @@ from collections import OrderedDict
 import torch
 from packaging import version
 from torch import Tensor, nn
+from typing import Any, Dict, List, Union, Optional, Tuple
+
+
+class RotaryEmbedding(nn.Module):
+    def __init__(self, dim, max_position_embeddings, base=10000, device=None):
+        """_summary_
+
+        Args:
+            dim (_type_): int(head_size)
+            max_position_embeddings (_type_): max_sentecne_length
+            base (int, optional): _description_. Defaults to 10000.
+            device (_type_, optional): _description_. Defaults to None.
+        """
+        super().__init__()
+        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
+        self.register_buffer("inv_freq", inv_freq)
+        self.max_seq_len_cached = max_position_embeddings
+        t = torch.arange(self.max_seq_len_cached, device=self.inv_freq.device, dtype=self.inv_freq.dtype)
+        freqs = torch.einsum("i,j->ij", t, self.inv_freq)
+
+        emb = torch.cat((freqs, freqs), dim=-1)
+        self.cos_cached = emb.cos()[None, None, :, :]
+        self.sin_cached = emb.sin()[None, None, :, :]
+
+    def forward(self, x, seq_len=None) -> Tuple[torch.Tensor, torch.Tensor]:
+        """
+
+        :param x: value
+        :param seq_len: sequence length
+        :return: cos and sin
+        """
+        if seq_len > self.max_seq_len_cached:
+            self.max_seq_len_cached = seq_len
+            t = torch.arange(self.max_seq_len_cached, device=x.device, dtype=self.inv_freq.dtype)
+            freqs = torch.einsum("i,j->ij", t, self.inv_freq)
+
+            emb = torch.cat((freqs, freqs), dim=-1).to(x.device)
+            self.cos_cached = emb.cos()[None, None, :, :]
+            self.sin_cached = emb.sin()[None, None, :, :]
+        return self.cos_cached[:seq_len, ...].to(x.device), self.sin_cached[:seq_len, ...].to(x.device)
 
 
 class HyperParameters(object):
@@ -224,7 +264,7 @@ linear_act = get_activation("linear")
 
 class Conv(M):
     def __init__(self, c1: int, c2: int, k: int = 1, s: int = 1, p: int = None, g: int = 1,
-                 activation: [str, torch.nn] = None,
+                 activation: Union[str, Any] = None,
                  form: int = -1):
         super(Conv, self).__init__()
         self.form = form
